"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
exports.unstable_getServerSession = unstable_getServerSession;

var _core = require("../core");

var _detectHost = require("../utils/detect-host");

var _utils = require("./utils");

async function NextAuthHandler(req, res, options) {
  var _options$secret, _options$jwt$secret, _options$jwt, _req$url, _req$body;

  (_options$secret = options.secret) !== null && _options$secret !== void 0 ? _options$secret : options.secret = (_options$jwt$secret = (_options$jwt = options.jwt) === null || _options$jwt === void 0 ? void 0 : _options$jwt.secret) !== null && _options$jwt$secret !== void 0 ? _options$jwt$secret : process.env.NEXTAUTH_SECRET;
  const host = (0, _detectHost.detectHost)(req.headers["x-forwarded-host"]);
  const url = new URL((_req$url = req.url) !== null && _req$url !== void 0 ? _req$url : "", host);
  const {
    status,
    headers,
    body
  } = await (0, _core.AuthHandler)(new Request(url, {
    headers: new Headers(req.headers),
    method: req.method,
    ...(0, _utils.getBody)(req)
  }), options);
  res.status(status);

  for (const [key, val] of headers.entries()) {
    const value = key === "set-cookie" ? val.split(",") : val;
    res.setHeader(key, value);
  }

  const redirect = headers.get("Location");

  if (((_req$body = req.body) === null || _req$body === void 0 ? void 0 : _req$body.json) === "true" && redirect) {
    res.removeHeader("Location");
    return res.json({
      url: redirect
    });
  }

  return res.send(body);
}

function NextAuth(...args) {
  if (args.length === 1) {
    return async (req, res) => await NextAuthHandler(req, res, args[0]);
  }

  return NextAuthHandler(args[0], args[1], args[2]);
}

var _default = NextAuth;
exports.default = _default;
let experimentalWarningShown = false;

async function unstable_getServerSession(...args) {
  var _options$secret2;

  if (!experimentalWarningShown && process.env.NODE_ENV !== "production") {
    console.warn("[next-auth][warn][EXPERIMENTAL_API]", "\n`unstable_getServerSession` is experimental and may be removed or changed in the future, as the name suggested.", `\nhttps://next-auth.js.org/configuration/nextjs#unstable_getServerSession}`, `\nhttps://next-auth.js.org/warnings#EXPERIMENTAL_API`);
    experimentalWarningShown = true;
  }

  const [req, res, options] = args;
  options.secret = (_options$secret2 = options.secret) !== null && _options$secret2 !== void 0 ? _options$secret2 : process.env.NEXTAUTH_SECRET;
  const host = (0, _detectHost.detectHost)(req.headers["x-forwarded-host"]);
  const url = new URL("/api/auth/session", host);
  const response = await (0, _core.AuthHandler)(new Request(url, {
    headers: req.headers
  }), options);
  const {
    status = 200,
    headers
  } = response;

  for (const [key, val] of headers.entries()) {
    const value = key === "set-cookie" ? val.split(",") : val;
    res.setHeader(key, value);
  }

  const data = await response.json();
  if (!data || !Object.keys(data).length) return null;
  if (status === 200) return data;
  throw new Error(data.message);
}